﻿// -------------------------------------------------------------------------------------------------------
// LICENSE INFORMATION
//
// - This software is licensed under the MIT shared source license.
// - The "official" source code for this project is maintained at http://mtcagent.codeplex.com
//
// Copyright (c) 2010 OpenNETCF Consulting
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of this software and 
// associated documentation files (the "Software"), to deal in the Software without restriction, 
// including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, 
// and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, 
// subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all copies or substantial 
// portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT 
// NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
// IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, 
// WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
// SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 
// -------------------------------------------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Reflection;

namespace OpenNETCF.MTConnect
{
    public class VirtualAgentConfiguration : IVirtualAgentConfiguration
    {
        public VirtualAgentConfiguration()
        {
            // set defaults
            var appPath = Path.GetFullPath(new Uri(Path.GetDirectoryName(Assembly.GetExecutingAssembly().GetName().CodeBase)).AbsolutePath);

            AutoRun = false;
            HostedAdapterFolder = Path.Combine(appPath, "Adapters");
            NewHostedAdapterFolder = Path.Combine(appPath, "NewAdapters");
            DeviceConfigurationFile = Path.Combine(appPath, "DeviceConfig.xml");

            CollectorFolder = null;
            ControllerFolder = null;
            ConnectionsFolder = null;

            EnableIPCService = false;
            EnableSHDRService = false;
            SHDRPort = 7878;
        }

        public bool AutoRun { get; internal set; }
        public string HostedAdapterFolder { get; internal set; }
        public string NewHostedAdapterFolder { get; internal set; }
        public string DeviceConfigurationFile { get; internal set; }

        public string CollectorFolder { get; set; }
        public string ControllerFolder { get; set; }
        public string ConnectionsFolder { get; set; }

        public bool EnableIPCService { get; internal set; }
        public bool EnableSHDRService { get; internal set; }
        public int SHDRPort { get; internal set; }
    }
}
