﻿// -------------------------------------------------------------------------------------------------------
// LICENSE INFORMATION
//
// - This software is licensed under the MIT shared source license.
// - The "official" source code for this project is maintained at http://mtcagent.codeplex.com
//
// Copyright (c) 2010 OpenNETCF Consulting
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of this software and 
// associated documentation files (the "Software"), to deal in the Software without restriction, 
// including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, 
// and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, 
// subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all copies or substantial 
// portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT 
// NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
// IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, 
// WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
// SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 
// -------------------------------------------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Configuration;
using System.Xml;
using System.Reflection;
using System.IO;

#if WindowsCE
using IConfigurationSectionHandler = OpenNETCF.Configuration.IConfigurationSectionHandler;
#endif

namespace OpenNETCF.MTConnect
{
    public class VirtualAgentConfigurationHandler : IConfigurationSectionHandler
    {
        public object Create(object parent, object configContext, XmlNode section)
        {
            var config = new VirtualAgentConfiguration();
            var appPath = Path.GetFullPath(new Uri(Path.GetDirectoryName(Assembly.GetExecutingAssembly().GetName().CodeBase)).AbsolutePath);

            try
            {
                config.HostedAdapterFolder = section.SelectSingleNode("HostedAdapterFolder").InnerText;
            }
            catch { /* use default */ }

            try
            {
                config.NewHostedAdapterFolder = section.SelectSingleNode("NewHostedAdapterFolder").InnerText;
            }
            catch { /* use default */ }

            try
            {
                config.AutoRun = Boolean.Parse(section.SelectSingleNode("AutoRun").InnerText);
            }
            catch { /* use default */ }

            try
            {
                config.DeviceConfigurationFile = Path.GetFullPath(section.SelectSingleNode("DeviceConfigurationFile").InnerText);
            }
            catch { /* use default */ }


            config.CollectorFolder = Path.Combine(appPath, "CollectorData");
            try
            {
                var node = section.SelectSingleNode("CollectorDataFolder");
                if (node != null)
                {
                    config.CollectorFolder = Path.GetFullPath(node.InnerText);
                }
            }
            catch { /* use default */ }

            config.ControllerFolder = Path.Combine(appPath, "Controllers");
            try
            {
                var node = section.SelectSingleNode("ControllersFolder");
                if (node != null)
                {
                    config.ControllerFolder = Path.GetFullPath(node.InnerText);
                }
            }
            catch { /* use default */ }

            config.ConnectionsFolder = Path.Combine(appPath, "Connections");
            try
            {
                var node = section.SelectSingleNode("ConnectionsFolder");
                if (node != null)
                {
                    config.ConnectionsFolder = Path.GetFullPath(node.InnerText);
                }
            }
            catch { /* use default */ }

            try
            {
                config.EnableIPCService = bool.Parse(section.SelectSingleNode("EnableIPCService").InnerText);
            }
            catch { /* use default */ }

            try
            {
                config.EnableSHDRService = bool.Parse(section.SelectSingleNode("EnableSHDRService").InnerText);
            }
            catch { /* use default */ }

            try
            {
                config.SHDRPort = int.Parse(section.SelectSingleNode("SHDRPort").InnerText);
            }
            catch { /* use default */ }

            return config;
        }
    }
}
