﻿// -------------------------------------------------------------------------------------------------------
// LICENSE INFORMATION
//
// - This software is licensed under the MIT shared source license.
// - The "official" source code for this project is maintained at http://mtconnect.codeplex.com
//
// Copyright (c) 2010 OpenNETCF Consulting
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of this software and 
// associated documentation files (the "Software"), to deal in the Software without restriction, 
// including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, 
// and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, 
// subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all copies or substantial 
// portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT 
// NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
// IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, 
// WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
// SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 
// -------------------------------------------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using OpenNETCF.MTConnect;
using OpenNETCF.Web.Server;
using OpenNETCF.Web;
using System.Net;

namespace OpenNETCF.MTConnect
{
    public class PadarnHost : HostBase
    {
        internal PadarnHost(Agent agent)
            : base(agent)
        {
        }

        public override void Start()
        {
        }

        public override bool IsClientConnected(object context)
        {
            var response = (HttpResponse)context;
            return response.IsClientConnected;
        }

        public override void ClearResponseHeaders(object context)
        {
            var response = (HttpResponse)context;
            response.ClearHeaders();
        }

        public override void SetResponseHeader(object context, string headerName, string value)
        {
            var response = (HttpResponse)context;
            if (string.Compare(headerName, "Content-Type", true) == 0)
            {
                response.ContentType = value;
            }
            else if (string.Compare(headerName, "Content-Length", true) == 0)
            {
                response.ContentLength = long.Parse(value);
            }
            else
            {
                response.AppendHeader(headerName, value);
            }
        }

        public override void SendResponse(string responseData, object context, bool flush)
        {
            var response = (HttpResponse)context;

            response.Write(responseData);
            if (flush)
            {
                try
                {
                response.Flush();
            }
                catch
                {
                    return;
                }
            }
        }
    }
}
