﻿// -------------------------------------------------------------------------------------------------------
// LICENSE INFORMATION
//
// - This software is licensed under the MIT shared source license.
// - The "official" source code for this project is maintained at http://mtcagent.codeplex.com
//
// Copyright (c) 2010 OpenNETCF Consulting
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of this software and 
// associated documentation files (the "Software"), to deal in the Software without restriction, 
// including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, 
// and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, 
// subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all copies or substantial 
// portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT 
// NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
// IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, 
// WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
// SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 
// -------------------------------------------------------------------------------------------------------

using System;
using System.Linq;
using System.Collections.Generic;
using System.Text;
using OpenNETCF.Web;
using System.Diagnostics;
using OpenNETCF.IoC;
using System.Xml.Linq;
using System.Threading;

namespace OpenNETCF.MTConnect
{
    public partial class VirtualAgentHandler
    {
        private void HandlePost(HttpContext context)
        {
            // if we're a POST to /agent, it's a schema update - we need to overwrite and handle that           
            var segments = GetRequestPathSegments(context);

            switch (segments.Length)
            {
                case 0:
                    SetConfiguration(context);
                    return;
                case 1:
                    switch (segments[0].ToLower())
                    {
                        case "data":
                            SetData(context);
                            return;
                        case "restart":
                            var agentService = RootWorkItem.Services.Get<IVirtualAgentService>();
                            agentService.Restart();
                            return;
                    }
                    break;
            }

            throw new HttpException(HttpErrorCode.MethodNotAllowed, "Unsupported POST");
        }

        private void SetConfiguration(HttpContext context)
        {
            var data = new byte[context.Request.InputStream.Length];
            var length = context.Request.InputStream.Read(data, 0, data.Length);
            var xml = Encoding.UTF8.GetString(data);
            MTConnectProvider.SetAdapterConfiguration(context.Request.UserHostAddress, xml);
        }

        private void SetData(HttpContext context)
        {
            var data = new byte[context.Request.InputStream.Length];
            var length = context.Request.InputStream.Read(data, 0, data.Length);
            var xml = Encoding.UTF8.GetString(data);

            var itemsNode = XElement.Parse(xml);

            if (itemsNode.Name != "DataItems")
            {
                throw new HttpException(HttpErrorCode.UnsupportedMediaType, "Unsupported POSTed entity");
            }

            foreach (var item in itemsNode.Elements("DataItem"))
            {
                if (item.Name == "DataItem")
                {
                    var idAttribute = item.Attribute(XmlAttributeName.DataItemID);

                    if (idAttribute == null)
                    {
                        throw new HttpException(HttpErrorCode.UnsupportedMediaType, "Unsupported POSTed entity");
                    }

                    var valueNode = item.Element(XmlNodeName.Value);

                    if (valueNode == null)
                    {
                        throw new HttpException(HttpErrorCode.UnsupportedMediaType, "Unsupported POSTed entity");
                    }

                    var value = valueNode.Value;

                    MTConnectProvider.SetDataByDataItemID(context.Request.UserHostAddress, idAttribute.Value, value);
                }
            }
        }
    }
}
