﻿// -------------------------------------------------------------------------------------------------------
// LICENSE INFORMATION
//
// - This software is licensed under the MIT shared source license.
// - The "official" source code for this project is maintained at http://mtcagent.codeplex.com
//
// Copyright (c) 2010 OpenNETCF Consulting
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of this software and 
// associated documentation files (the "Software"), to deal in the Software without restriction, 
// including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, 
// and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, 
// subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all copies or substantial 
// portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT 
// NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
// IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, 
// WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
// SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 
// -------------------------------------------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using OpenNETCF.IoC;
using System.Threading;
using System.Diagnostics;

namespace OpenNETCF.MTConnect.VirtualAgent
{
    public class Module : ModuleInit
    {
        private HostedAdapterService HostedAdapterService { get; set; }
        private IConfigurationService ConfigurationService { get; set; }
        private IMTConnectProvider Provider { get; set; }

        public override void AddServices()
        {
            RootWorkItem.Services.AddNew<LogService, ILogService>();
            ConfigurationService = RootWorkItem.Services.AddNew<ConfigurationService, IConfigurationService>();
            Provider = RootWorkItem.Services.AddNew<PadarnService, IMTConnectProvider>();
            RootWorkItem.Services.AddNew<AgentService, IVirtualAgentService>();
            RootWorkItem.Services.AddNew<InstanceService, IInstanceService>();
            HostedAdapterService = RootWorkItem.Services.AddNew<HostedAdapterService, IHostedAdapterService>();

            if (ConfigurationService.AgentConfiguration.EnableIPCService)
            {
            if (Environment.OSVersion.Platform == PlatformID.WinCE)
            {
                RootWorkItem.Services.AddNew<IPCService>();
            }
        }

            if (ConfigurationService.AgentConfiguration.EnableSHDRService)
            {
                RootWorkItem.Services.AddNew<SHDRAdapterService>();
            }
        }

        public override void OnContainerLoadComplete()
        {
            // let the adapters initialize now that all modules are loaded
            HostedAdapterService.InitializeAdapters();

            if (ConfigurationService.AgentConfiguration.AutoRun)
            {
                // let all of the Adapters initialize before we fire up the web server
                ThreadPool.QueueUserWorkItem(delegate
                {
                    Provider.Start();
                });
            }
        }
    }
}
